unit bi_ctrls;

interface

uses Windows, Messages, Classes, Controls, Forms, Graphics,
  ExtCtrls, buttons, bi_timer, sysutils;

type

  TBI_Loupe = class(TGraphicControl)
  private
    _timer:tbi_timer;
    x,y                  : integer;
    DC,dcpuffer,puffer   : hdc;
    fzoom                : double;
    factive              : boolean;
    mybuffer             : tbitmap;
    finterval            : integer;
    fbeveli              : tpanelbevel;
    fbevelo              : tpanelbevel;
    fbevelwidth          : tbevelwidth;
    fdesign              : boolean;
    procedure _mypaint(sender:tobject);
    procedure resized;
    procedure setzoom(value:double);
    procedure setinterval(value:integer);
    procedure setactive(value:boolean);
    procedure setdesign(value:boolean);
    procedure setbeveli(value:tpanelbevel);
    procedure setbevelo(value:tpanelbevel);
    procedure setbevelwidth(value:tbevelwidth);
    procedure paintcrosshairs;
  protected
  procedure paint; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure savetofile(s1:string);
  published
    property ZoomLevel:double read fzoom write setzoom;
    property Interval:integer read finterval write setinterval;
    property Active:boolean read factive write setactive;
    property BevelInner:tpanelbevel read fbeveli write setbeveli;
    property BevelOuter:tpanelbevel read fbevelo write setbevelo;
    property Bevelwidth:tbevelwidth read fbevelwidth write setbevelwidth;
    property DrawDesigning:boolean read fdesign write setdesign;
    property Visible;
    property OnClick;
    property OnDblClick;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property ShowHint;
    property ParentFont;
    property Align;
    property ParentColor;
    property ParentShowHint;
    property PopupMenu;
  end;

  TWebGauge = class(TGraphicControl)
  private
    FMinValue: Longint;
    FMaxValue: Longint;
    FCurValue: Longint;
    FForeColor: TColor;
    FBackColor: TColor;
    FWaiting  : Boolean;
    FShowText: Boolean;
    timer:tbi_timer;
    fbeveli              : tpanelbevel;
    fbevelo              : tpanelbevel;
    fbevelwidth          : tbevelwidth;
    step                 : integer;
    mright:boolean;
    colors:array[0..32]of tcolor;
    procedure SetShowText(Value: Boolean);
    procedure SetForeColor(Value: TColor);
    procedure SetBackColor(Value: TColor);
    procedure SetMinValue(Value: Longint);
    procedure SetMaxValue(Value: Longint);
    procedure SetProgress(Value: Longint);
    procedure SetWaiting(Value: Boolean);
    procedure calccolors;
    procedure timertick(sender:tobject);
    procedure setbeveli(value:tpanelbevel);
    procedure setbevelo(value:tpanelbevel);
    procedure setbevelwidth(value:tbevelwidth);
    function GetPercentDone: Longint;
  protected
    procedure Paint; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure AddProgress(Value: Longint);
    property PercentDone: Longint read GetPercentDone;
  published
    property Align;
    property Color;
    property Enabled;
    property BevelInner:tpanelbevel read fbeveli write setbeveli;
    property BevelOuter:tpanelbevel read fbevelo write setbevelo;
    property Bevelwidth:tbevelwidth read fbevelwidth write setbevelwidth;
    property ShowText: Boolean read FShowText write SetShowText default True;
    property Font;
    property ForeColor: TColor read FForeColor write SetForeColor default clBlack;
    property BackColor: TColor read FBackColor write SetBackColor default clWhite;
    property MinValue: Longint read FMinValue write SetMinValue default 0;
    property MaxValue: Longint read FMaxValue write SetMaxValue default 100;
    property Waiting: Boolean read Fwaiting write Setwaiting default false;
    property ParentColor;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property Progress: Longint read FCurValue write SetProgress;
    property ShowHint;
    property Visible;
  end;

//<-------------------------------------------------------------------TBI_Arrow
  TArrowDirection = (adRight,adLeft,adUp,adDown);
  TBI_Arrow = class(tgraphiccontrol)
  private
  FPen: TPen;
  FBrush: TBrush;
  FDir:TArrowDirection;
  procedure SetBrush(Value: TBrush);
  procedure SetPen(Value: TPen);
  procedure SetDir(Value: TArrowDirection);
  protected
  procedure paint;override;
  public
  constructor create(aowner:tcomponent);override;
  destructor destroy;override;
  published
  property Width;
  property Height;
  property Pen: TPen  read FPen write SetPen;
  property Brush: TBrush  read FBrush write SetBrush;
  property Direction:TArrowDirection read fdir write Setdir;
  property OnDragDrop;
  property OnDragOver;
  property OnEndDrag;
  property OnMouseDown;
  property OnMouseMove;
  property OnMouseUp;
  end;

  TBI_Bevel = class(TGraphicControl)
  private
    FStyle: TBevelStyle;
    FShape: TBevelShape;
    FColor: Tcolor;
    fpstyle: tpenstyle;
    ftransparent:boolean;
    dcolor,lcolor:tcolor;
    procedure SetStyle(Value: TBevelStyle);
    procedure SetShape(Value: TBevelShape);
    procedure settransparent(Value: boolean);
    procedure setpenstyle(Value: tpenstyle);
    procedure setcolor(Value: TColor);
  protected
    procedure Paint; override;
  public
    constructor Create(AOwner: TComponent); override;
  published
    property Align;
    property ParentShowHint;
    property Shape: TBevelShape read FShape write SetShape default bsBox;
    property ShowHint;
    property PenStyle:tpenstyle read fpstyle write setpenstyle;
    property Transparent:boolean read ftransparent write settransparent;
    Property Color:tcolor read fcolor write setcolor;
    property Style: TBevelStyle read FStyle write SetStyle default bsLowered;
    property Visible;
  end;

//<-------------------------Wallpaper---------------------------------------------
 TBI_Wallpaper = class;

  TWallpaperMode   = (wpmNormal,wpmFaded, wpmTransparent,wpmNotebook,wpmSawtooth);
  TnonBMPOptions = class(TPersistent)
                     private
                     fowner:TBI_Wallpaper;
                     fpcolor,fback,flcolor,fhole:tcolor;
                     fxoffset,fyoffset:integer;
                     procedure setpcolor(value:tcolor);
                     procedure setback(value:tcolor);
                     procedure setlcolor(value:tcolor);
                     procedure sethole(value:tcolor);
                     procedure setxoffset(value:integer);
                     procedure setyoffset(value:integer);
                     published
                     property PaperColor:tcolor read fpcolor write setpcolor;
                     property BackGround:tcolor read fback write setback;
                     property Highlight:tcolor read flcolor write setlcolor;
                     property Shadow:tcolor read fhole write sethole;
                     property XOffset:integer read fxoffset write setxoffset;
                     property YOffset:integer read fyoffset write setyoffset;
                     end;

  TBI_Wallpaper=class(tgraphiccontrol)
  protected
    FImage           : TBitmap;
    FInternalImage   : TBitmap;
    FMode            : TWallpaperMode;
    ftranscolor      : TColor;
    FFadeColor       : TColor;
    FNBOptions       : TnonBMPOptions;
    procedure Paint; override;
    procedure SetImage(Value: TBitMap);
    procedure renderinternalimage;
    procedure SetMode(Value: TWallpaperMode);
    procedure SetTransColor(Value: TColor);
    procedure SetFadeColor(Value: TColor);
    procedure SetNBOptions(Value: TnonBMPOptions);
    procedure loaded;override;
    procedure CMSysColorChange(var Message: TMessage); message CM_SYSCOLORCHANGE;
  public
    constructor Create(AOwner: TComponent); override;
    destructor  Destroy; override;
  published
    property Image    : TBitMap read fImage write SetImage;
    property TransColor: TColor read ftranscolor write SetTransColor;
    property FadeColor: TColor read FFadeColor write SetFadeColor default clBtnHighlight;
    property Mode : TWallpaperMode read FMode write SetMode ;
    property NonBMPOptions : TnonBMPOptions read fNBOptions write setnboptions stored true;
    property Height default 100;
    property Width  default 100;
    property Visible;
    property Align default alClient;
    property Enabled;
    property ParentShowHint;
    property ShowHint;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
  end;


implementation

uses consts,bi_procs;

type myrgb=record r:byte;
                g:byte;
                b:byte; end;

//<--------------------------Webgauge
constructor TWebGauge.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ControlStyle := ControlStyle + [csFramed, csOpaque];
  FMinValue := 0;
  FMaxValue := 100;
  FCurValue := 0;
  FWaiting  := false;
  FShowText := True;
  FForeColor := clwindowtext;
  FBackColor := clbtnface;
  Fwaiting:=false;
  Width := 200;
  Height := 20;
  step:=0;
  timer:=TBI_timer.create(self);
  timer.interval:=55;
  timer.enabled:=false;
  timer.ontimer:=timertick;
  timer.threadpriority:=tplower;
  fbeveli:=bvnone;
  fbevelwidth:=1;
  mright:=true;
  fbevelo:=bvnone;
end;

destructor TWebGauge.destroy;
begin
timer.enabled:=false;
timer.free;
inherited destroy;
end;

function TWebGauge.GetPercentDone: Longint;
begin
  if FMaxValue - FMinValue = 0 then result := 0
  else Result := Trunc(((FCurValue - FMinValue)*100)/FMaxValue - FMinValue)
end;

procedure TWebGauge.SetShowText(Value: Boolean);
begin
    FShowText := Value;
    invalidate;
end;

procedure TWebGauge.timertick(sender:tobject);
begin
     if mright then
     begin
     inc(step);
     if step+1>(width/2) then
     mright:=false;
     end
     else
     begin
     dec(step);
     if step=0 then
     mright:=true;
     end;
     repaint;
end;


procedure TWebGauge.SetForeColor(Value: TColor);
begin
    FForeColor := Value;
    Refresh;
end;

procedure TWebGauge.SetBackColor(Value: TColor);
begin
    FBackColor := Value;
    Refresh;
end;

procedure TWebGauge.SetMinValue(Value: Longint);
begin
  if Value <> FMinValue then
  begin
    if Value > FMaxValue then
      value:=fmaxvalue;
    FMinValue := Value;
    if FCurValue < Value then FCurValue := Value;
    Refresh;
  end;
end;

procedure TWebGauge.SetMaxValue(Value: Longint);
begin
  if Value <> FMaxValue then
  begin
    if Value < FMinValue then
      raise EInvalidOperation.CreateResFmt(SOutOfRange, [FMinValue + 1, MaxInt]);
    FMaxValue := Value;
    if FCurValue > Value then FCurValue := Value;
    Refresh;
  end;
end;

procedure TWebGauge.SetProgress(Value: Longint);
var
  TempPercent: Longint;
begin
  TempPercent := GetPercentDone;
  if Value < FMinValue then
    Value := FMinValue
  else if Value > FMaxValue then
    Value := FMaxValue;  {execption auslsen d2 vs d3???}
  if FCurValue <> Value then
  begin
    FCurValue := Value;
    if TempPercent <> GetPercentDone then
      Refresh;
  end;
end;

procedure TWebGauge.SetWaiting(Value: Boolean);
begin
if value<>fwaiting then
   begin
   fwaiting:=value;
   timer.enabled:=value;
   calccolors;
   invalidate;
   end
end;

procedure TWebGauge.AddProgress(Value: Longint);
begin
  Progress := FCurValue + Value;
  Refresh;
end;

 procedure twebgauge.calccolors;
 var x:integer;
     _a,_b,_c:myrgb;
 begin
 _a.r:=getrvalue(colortorgb(backcolor));
 _a.g:=getgvalue(colortorgb(backcolor));
 _a.b:=getbvalue(colortorgb(backcolor));


 _b.r:=getrvalue(colortorgb(forecolor));
 _b.g:=getgvalue(colortorgb(forecolor));
 _b.b:=getbvalue(colortorgb(forecolor));
 for x:=0 to 32 do
 begin
  if _a.r<_b.r then
_c.r:=_a.r+round(((_b.r-_a.r) / 32)*x)
 else _c.r:=_a.r-round(((_a.r-_b.r) / 32)*x);
 if _a.g<_b.g then
       _c.g:=_a.g+round(((_b.g-_a.g) / 32)*x)
 else _c.g:=_a.g-round(((_a.g-_b.g) / 32)*x);
 if _a.b<_b.b then
       _c.b:=_a.b+round(((_b.b-_a.b) / 32)*x)
 else _c.b:=_a.b-round(((_a.b-_b.b) / 32)*x);
 colors[x]:=rgb(_c.r,_c.g,_c.b);
 end;
 end;

procedure TWebGauge.Paint;
var _rect:trect;
    tmp,x:integer;
    tmp2:tbitmap;
    tmp3:tbitmap;
    S1:string;
    topcolor,bottomcolor:tcolor;

begin
tmp2:=tbitmap.create;
tmp2.width:=width;
tmp2.height:=height;

if waiting then
begin
     with tmp2.canvas do
     begin
     brush.color:=fbackcolor;
     brush.style:=bssolid;
     pen.color:=fbackcolor;
     rectangle(0,0,width,height);
     pen.style:=pssolid;
     for x:=0 to width div 2 do
         begin
         if mright then
         pen.color:=colors[trunc((x/(width/2))*32)]
         else
         pen.color:=colors[32-trunc((x/(width/2))*32)];
         moveto(x+step,0);
         lineto(x+step,height);
         end;
     end;
end
else
begin
tmp3:=tbitmap.create;
tmp3.width:=width;
tmp3.height:=height;
   with tmp2.canvas do
        begin
        _Rect := clientrect;
        brush.color:=fbackcolor;
        brush.style:=bssolid;
        pen.color:=fbackcolor;
        rectangle(0,0,width,height);
        brush.color:=fforecolor;
        tmp:=Trunc(percentdone*(width * 0.01));
        rectangle(0,0,tmp,height);

        S1 := Format('%d%%', [PercentDone]);
        brush.style:=bsclear;
        //adjust:=;

        end;
        with tmp3.canvas do
        begin
        font:=self.font;
        brush.color:=clblack;
        brush.style:=bssolid;
        rectangle(0,0,width,height);
        font.color:=clwhite;
        brush.style:=bsclear;
        _rect:=rect(5,(height-textheight(s1))div 2,width,height);
        DrawText(Handle, PChar(s1), Length(s1), _Rect, dt_center);
        //textrect(_rect,0,0,s1);
        end;
tmp2.canvas.copymode:=cmsrcinvert;
tmp2.canvas.draw(0,0,tmp3);
tmp3.free;
end;

  _Rect := ClientRect;

  if BevelOuter <> bvNone then
  begin
    TopColor := clBtnHighlight;
    if BevelOuter = bvLowered then TopColor := clBtnShadow;
    BottomColor := clBtnShadow;
    if BevelOuter = bvLowered then BottomColor := clBtnHighlight;
    Frame3D(tmp2.Canvas, _Rect, TopColor, BottomColor, BevelWidth);
  end;
  if BevelInner <> bvNone then
  begin
    TopColor := clBtnHighlight;
    if BevelInner = bvLowered then TopColor := clBtnShadow;
    BottomColor := clBtnShadow;
    if BevelInner = bvLowered then BottomColor := clBtnHighlight;
    Frame3D(tmp2.Canvas, _Rect, TopColor, BottomColor, BevelWidth);
  end;

canvas.draw(0,0,tmp2);
tmp2.free;
end;

procedure twebgauge.setbeveli(value:tpanelbevel);
begin
fbeveli:=value;
invalidate;
end;

procedure twebgauge.setbevelo(value:tpanelbevel);
begin
fbevelo:=value;
invalidate;
end;

procedure twebgauge.setbevelwidth(value:tbevelwidth);
begin
fbevelwidth:=value;
invalidate;
end;
//<----------------------------------------------Loupe

constructor TBI_loupe.Create(AOwner: TComponent);
begin
inherited Create(AOwner);
ControlStyle := [csclickevents, csCaptureMouse, csOpaque, csDoubleClicks];
mybuffer:=tbitmap.create;
mybuffer.width:=100;
mybuffer.height:=100;
_timer:=TBI_timer.create(self);
_timer.interval:=100;
finterval:=100;
width:=100;
height:=100;
fbeveli:=bvRaised;
fbevelwidth:=1;
fbevelo:=bvlowered;
fzoom:=1;
fdesign:=false;
dc := createdc('DISPLAY',Nil,Nil,Nil);
dcpuffer := createcompatibledc(dc);
x:=round(width / fzoom);
y:=round(height / fzoom);
if puffer<>0 then deletedc(puffer);
puffer := createcompatiblebitmap(dc,x,y);
selectobject(dcpuffer,puffer);
_timer.enabled:=false;
active:=false;
_timer.ontimer:=_mypaint;
end;

destructor tBI_loupe.destroy;
begin
_timer.enabled:=false;
deletedc(puffer);
deletedc(dcpuffer);
deletedc(dc);
_timer.free;
mybuffer.free;
inherited destroy;
end;

procedure tBI_loupe._mypaint(Sender: TObject);
var  position             : tpoint;
begin
if (csdesigning in componentstate) and (not fdesign) then exit;
getcursorpos(position);
  bitblt (dcpuffer,0,0,width,height,dc,
         position.x-(x div 2),position.y-(y div 2),SRCCOPY);
  stretchblt (mybuffer.canvas.handle,0,0,width,height,
         DCPuffer,0,0,x,y,SRCCOPY);
  repaint;

end;

procedure TBI_loupe.resized;
begin
mybuffer.width:=width;
mybuffer.height:=height;
x:=round(width / fzoom);
y:=round(height / fzoom);
if puffer<>0 then deletedc(puffer);
puffer := createcompatiblebitmap(dc,x,y);
selectobject(dcpuffer,puffer);
//paintcrosshairs(self);
end;

procedure TBI_loupe.setzoom(value:double);
begin
if value>0 then fzoom:=value else fzoom:=1;
x:=round(width / fzoom);
y:=round(height / fzoom);
if puffer<>0 then deletedc(puffer);
puffer := createcompatiblebitmap(dc,x,y);
selectobject(dcpuffer,puffer);
end;

procedure TBI_loupe.setinterval(value:integer);
begin
_timer.enabled:=false;
finterval:=value;
_timer.interval:=finterval;
_timer.enabled:=true;
end;

procedure TBI_loupe.setactive(value:boolean);
begin
factive:=value;
_timer.enabled:=factive;
end;

procedure TBI_loupe.paint;
var _rect:trect;
topcolor,bottomcolor:tcolor;
begin
if (mybuffer.width<>width) or (mybuffer.height<>height) then resized;
  _Rect := GetClientRect;
  if BevelOuter <> bvNone then
  begin
    TopColor := clBtnHighlight;
    if BevelOuter = bvLowered then TopColor := clBtnShadow;
    BottomColor := clBtnShadow;
    if BevelOuter = bvLowered then BottomColor := clBtnHighlight;
    Frame3D(mybuffer.Canvas, _Rect, TopColor, BottomColor, BevelWidth);
  end;
  if BevelInner <> bvNone then
  begin
    TopColor := clBtnHighlight;
    if BevelInner = bvLowered then TopColor := clBtnShadow;
    BottomColor := clBtnShadow;
    if BevelInner = bvLowered then BottomColor := clBtnHighlight;
    Frame3D(mybuffer.Canvas, _Rect, TopColor, BottomColor, BevelWidth);
  end;
bitblt(canvas.handle,canvas.cliprect.left,canvas.cliprect.top,
canvas.cliprect.right,canvas.cliprect.bottom,mybuffer.canvas.handle,
canvas.cliprect.left,canvas.cliprect.top,srccopy);
end;

procedure TBI_loupe.paintcrosshairs;
begin
{bevel1.left:=(image1.width div 2)-1;
bevel1.top:=(image1.height div 2)-10;
bevel2.left:=(image1.width div 2)-10;
bevel2.top:=(image1.height div 2)-1;}
end;

procedure TBI_loupe.savetofile(s1:string);
begin
_timer.enabled:=false;
mybuffer.savetofile(s1);
_timer.enabled:=factive;
end;

procedure TBI_loupe.setbeveli(value:tpanelbevel);
begin
fbeveli:=value;
invalidate;
end;

procedure TBI_loupe.setbevelo(value:tpanelbevel);
begin
fbevelo:=value;
invalidate;
end;

procedure TBI_loupe.setdesign(value:boolean);
begin
fdesign:=value;
invalidate;
end;

procedure TBI_loupe.setbevelwidth(value:tbevelwidth);
begin
fbevelwidth:=value;
invalidate;
end;

//<----------------------------------------------------TBI_Bevel

constructor TBI_bevel.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ControlStyle := ControlStyle + [csReplicatable];
  FStyle := bsLowered;
  FShape := bsBox;
  Fpstyle:=pssolid;
  ftransparent:=true;
  Fcolor:= clbtnface;
  dcolor:=clbtnshadow;
  lcolor:=clbtnhighlight;
  Width := 50;
  Height := 50;
end;

procedure TBI_bevel.SetStyle(Value: TBevelStyle);
begin
  if Value <> FStyle then
  begin
    FStyle := Value;
    Invalidate;
  end;
end;

procedure TBI_bevel.SetShape(Value: tbevelShape);
begin
  if Value <> FShape then
  begin
    FShape := Value;
    Invalidate;
  end;
end;

procedure TBI_bevel.Paint;
var
  Color1, Color2: TColor;
  Temp: TColor;

  procedure BevelRect(const R: TRect);
  begin
    with Canvas do
    begin
      Pen.Color := Color1;
      Pen.Style := fpstyle;
      PolyLine([Point(R.Left, R.Bottom), Point(R.Left, R.Top),
        Point(R.Right, R.Top)]);
      Pen.Color := Color2;
      PolyLine([Point(R.Right, R.Top), Point(R.Right, R.Bottom),
        Point(R.Left, R.Bottom)]);
    end;
  end;

  procedure DoLineExt(C: TColor; X1, Y1, X2, Y2: Integer);
  begin
    with Canvas do
    begin
      Pen.Style := fpstyle;
      Pen.Color := c;
      MoveTo(X1, Y1);
      LineTo(X2, Y2);
    end;
  end;

begin
  with Canvas do
  begin
    if not ftransparent then
       begin
       brush.style:=bssolid;
       brush.color:=fcolor;
       rectangle(0,0,width,height);
       end;
    Pen.Width := 1;

    if FStyle = bsLowered then
    begin
      Color1 := dcolor;
      Color2 := lcolor;
    end
    else
    begin
      Color1 := lcolor;
      Color2 := dcolor;
    end;

    case FShape of
      bsBox: BevelRect(Rect(0, 0, Width - 1, Height - 1));
      bsFrame:
        begin
          Temp := Color1;
          Color1 := Color2;
          BevelRect(Rect(1, 1, Width - 1, Height - 1));
          Color2 := Temp;
          Color1 := Temp;
          BevelRect(Rect(0, 0, Width - 2, Height - 2));
        end;
      bsTopLine:
        begin
          dolineext(Color1, 0, 0, Width, 0);
          dolineext(Color2, 0, 1, Width, 1);
        end;
      bsBottomLine:
        begin
          dolineext(Color1, 0, Height - 2, Width, Height - 2);
          dolineext(Color2, 0, Height - 1, Width, Height - 1);
        end;
      bsLeftLine:
        begin
          dolineext(Color1, 0, 0, 0, Height);
          dolineext(Color2, 1, 0, 1, Height);
        end;
      bsRightLine:
        begin
          dolineext(Color1, Width - 2, 0, Width - 2, Height);
          dolineext(Color2, Width - 1, 0, Width - 1, Height);
        end;
    end;
  end;
end;

procedure TBI_bevel.settransparent(Value: boolean);
begin
if value<>ftransparent then
   begin
   ftransparent:=value;
   invalidate;
   end;
end;

procedure TBI_bevel.setpenstyle(Value: tpenstyle);
begin
if value<>fpstyle then
   begin
   fpstyle:=value;
   invalidate;
   end;
end;

procedure TBI_bevel.setcolor(Value: TColor);
            function lightencolor(c:tcolor):tcolor;
  var _c,_nc:myrgb;
  begin
  _c.r:=getrvalue(colortorgb(c));
  _c.g:=getgvalue(colortorgb(c));
  _c.b:=getbvalue(colortorgb(c));

  _nc.r:=round(_c.r+25);
  if _nc.r>255 then _nc.r:=255;
  _nc.g:=round(_c.g+25);
  if _nc.g>255 then _nc.g:=255;
  _nc.b:=round(_c.b+25);
  if _nc.b>255 then _nc.b:=255;

  c:=rgb(_nc.r,_nc.g,_nc.b);
  result:=c//getnearestcolor(canvas.handle,C)
  end;

  function darkencolor(c:tcolor):tcolor;
  var _c,_nc:myrgb;
  begin
  _c.r:=getrvalue(colortorgb(c));
  _c.g:=getgvalue(colortorgb(c));
  _c.b:=getbvalue(colortorgb(c));

  _nc.r:=round(_c.r-25);
  if _nc.r<0 then _nc.r:=0;
  _nc.g:=round(_c.g-25);
  if _nc.g<0 then _nc.g:=0;
  _nc.b:=round(_c.b-25);
  if _nc.b<0 then _nc.b:=0;

  c:=rgb(_nc.r,_nc.g,_nc.b);
  result:=c
  end;
begin
if value<>fcolor then
   begin
   fcolor:=value;
   if color<>clbtnface then
   begin
   dcolor:=darkencolor(value);
   lcolor:=lightencolor(value);
   end
   else
   begin
   dcolor:=clbtnshadow;
   lcolor:=clbtnhighlight;
   end;
   invalidate;
   end;
end;

//<<----------------------------TBI_Arrow
constructor TBI_Arrow.create(aowner:tcomponent);
begin
inherited create(aowner);
width:=50;
height:=50;
color:=clblack;
FBrush := TBrush.Create;
fbrush.color:=clblack;
FPen := TPen.Create;
fdir:=adDown;
end;

destructor TBI_Arrow.Destroy;
begin
fpen.free;
fbrush.free;
inherited destroy;
end;

procedure TBI_Arrow.SetBrush(Value: TBrush);
begin
  FBrush.Assign(Value);
  invalidate;
end;

procedure TBI_Arrow.SetPen(Value: TPen);
begin
  FPen.Assign(Value);
  invalidate;
end;

procedure TBI_Arrow.Setdir(Value: tarrowdirection);
begin
  if value<>fdir then
  begin
  fdir:=value;
  invalidate;
  end;
end;

procedure TBI_Arrow.paint;
var _points:array[1..3] of tpoint;
    x1,x2,x3,x4,x5,y1,y2,y3,y4,y5:word;
begin
x1:=0; y2:=0; y1:=0; x2:=0; x3:=0; x4:=0; x5:=0; y3:=0; y4:=0; y5:=0;
{stop compiler warnings}
case fdir of
     adDown : begin
              x1:=width div 3;
              x2:=0;
              x3:=width;
              x4:=width div 2;
              x5:=x1*2;
              y1:=height div 2;
              y2:=0;
              y3:=height;
              y5:=y1;
              y4:=y1;
              end;
     adUp : begin
              x1:=width div 3;
              x2:=0;
              x3:=width;
              x4:=width div 2;
              x5:=x1*2;
              y1:=height div 2;
              y2:=height;
              y3:=0;
              y5:=y1;
              y4:=y1;
              end;
     adLeft: begin
              x1:=width div 2;
              y2:=height div 3;
              x5:=width;
              y1:=y2*2;
              x2:=x1;
              y4:=0;
              x3:=x2;
              y5:=height;
              x4:=0;
              y3:=height div 2;
              end;
     adRight: begin
              x1:=0;
              y2:=height div 3;
              x5:=width div 2;
              y1:=y2*2;
              x2:=width div 2;
              y4:=0;
              x3:=x2;
              y5:=height;
              x4:=width;
              y3:=height div 2;
              end;
     end;
with canvas do
begin
brush.color:=(parent as twincontrol).brush.color;
brush.style:=bsclear;
fillrect(clientrect);
brush:=fbrush;
pen:=fpen;
rectangle(x1, y2,x5,y1);
_points[1]:=point(x2,y4);
_points[2]:=point(x3,y5);
_points[3]:=point(x4,y3);
polygon(_points);
end;

end;

//<<--------------------Wallpaper-------------------------------------------------

constructor TBI_Wallpaper.Create(AOwner: TComponent);
begin
inherited Create(AOwner);
controlstyle:=controlstyle + [csopaque];
FInternalImage:=TBitMap.Create;
FImage:=TBitMap.Create;
ffadecolor:=clBtnHighlight;
ftranscolor:=clblack;
FMode:=wpmsawtooth;
width:=100; height:=100;
Align:=alClient;
fnboptions:=TnonBMPOptions.create;
fnboptions.fowner:=self;
fnboptions.fpcolor:=clwhite;
fnboptions.fback:=clbtnface;
fnboptions.flcolor:=clsilver;
fnboptions.fhole:=clgray;
fnboptions.fxoffset:=20;
fnboptions.yoffset:=20;
end;

destructor TBI_Wallpaper.Destroy;
begin
FImage.Free;
FInternalImage.Free;
fnboptions.free;
inherited Destroy;
end;

procedure TBI_Wallpaper.SetImage(Value: tbitmap);
begin
  if Assigned(Value) then
  begin
    ftranscolor := Value.TransparentColor;
    fimage.Assign(Value);
    canvas.brush.color:=(parent as twincontrol).brush.color;
    canvas.rectangle(0,0,width,height);
    renderinternalimage;
    invalidate;
  end
  else
  begin
    FImage.Height := 0;
    FImage.Width  := 0;
    renderinternalimage;
    invalidate;
  end;
end;

procedure TBI_Wallpaper.CMSysColorChange(var Message: TMessage);
begin
  renderinternalimage;
  invalidate;
end;

procedure tbi_wallpaper.SetMode(Value: TWallpaperMode);
begin
if value<>fmode then
   begin
   fmode:=value;
   renderinternalimage;
   invalidate;
   end;
end;

procedure tbi_wallpaper.SetNBOptions(Value: TnonBMPOptions);
begin
fNBOptions.assign(value);
renderinternalimage;
invalidate;
end;

procedure tbi_wallpaper.renderinternalimage;
var tmp:tbitmap;
    sawpoints:array[1..3] of tpoint;
    h,w:integer;
begin
 case fmode of
      wpmnormal,
      wpmtransparent : finternalimage.assign(fimage);
      wpmfaded       : begin
                       tmp:=tbitmap.create; tmp.width:=fimage.width; tmp.height:=fimage.height;
                       tmp.monochrome:=true; tmp.assign(fimage);
                       finternalimage.assign(tmp);
                       FinternalImage.Canvas.Brush.Color := (Parent as TWinControl).Brush.Color;
                       finternalimage.canvas.fillrect(rect(0,0,tmp.width,tmp.height));
                       DrawBitmapTransparent(finternalimage.Canvas,
                                  0, 0, tmp, clblack );
                       FinternalImage.Canvas.Brush.Color := ffadecolor;
                       FinternalImage.Canvas.BrushCopy
                       (Rect (0, 0, tmp.Width, tmp.Height), FinternalImage,
                       Rect (0, 0, tmp.Width, tmp.Height), clwhite);
                       tmp.free;
                       end;
      wpmnotebook     :begin
                       finternalimage.width:=fnboptions.xoffset;
                       finternalimage.height:=fnboptions.yoffset;
                       with finternalimage.canvas,fnboptions do
                            begin
                            pen.width:=1;
                            brush.color:=fnboptions.papercolor;
                            fillrect(rect(0,0,finternalimage.width,finternalimage.height));
                            pen.color:=fnboptions.shadow;
                            brush.color:=fnboptions.shadow;
                            ellipse(0,0,xoffset,yoffset);
                            brush.style:=bsclear;
                            pen.width:=3;
                            ellipse(1,1,xoffset,yoffset);
                            pen.width:=1;
                            pen.style:=psclear;
                            brush.style:=bssolid;
                            brush.color:=fnboptions.Background;
                            ellipse(2,2,xoffset,yoffset);
                            end;
                       end;
      wpmsawtooth     :begin
                       finternalimage.width:=fnboptions.xoffset;
                       finternalimage.height:=fnboptions.yoffset;
                       h:=fnboptions.yoffset ;
                       w:=fnboptions.xoffset;
                       with finternalimage.canvas,fnboptions do
                            begin
                            pen.width:=1;
                            brush.color:=papercolor;
                            pen.color:=brush.color;
                            fillrect(rect(0,0,w,h));
                            sawpoints[1]:=point(0,0);
                            sawpoints[2]:=point(w,0);
                            sawpoints[3]:=point(0,h);
                            brush.color:=shadow;
                            pen.color:=brush.color;
                            polygon(sawpoints);
                            sawpoints[1]:=point(0,0);
                            sawpoints[2]:=point(w-5,0);
                            sawpoints[3]:=point(0,h-5);
                            brush.color:=highlight;
                            pen.color:=brush.color;
                            polygon(sawpoints);
                            end;
                       end;
 end;
end;

procedure tbi_wallpaper.settranscolor(value:tcolor);
begin
if value<>ftranscolor then
   begin
   ftranscolor:=value;
   invalidate;
   end;
end;

procedure tbi_wallpaper.setfadecolor(value:tcolor);
begin
if value<>ffadecolor then
   begin
   ffadecolor:=value;
   renderinternalimage;
   invalidate;
   end;
end;

procedure tbi_wallpaper.loaded;
begin
inherited loaded;
renderinternalimage;
invalidate;
end;

procedure tbi_wallpaper.paint;
var buffer:tbitmap;
    x,y,w,h:integer;
begin
buffer:=tbitmap.create; buffer.width:=width; buffer.height:=height;
if finternalimage.width>0 then
case fmode of
     wpmNormal    ,
     wpmfaded      : begin
                    with finternalimage do
                         begin w:=width; h:=height; end;
                         y:=0;
                     while y<height do
                           begin
                           x:=0;
                           while x<width do
                                  begin
                                  buffer.canvas.draw(x,y,finternalimage);
                                  inc(x,w);
                                  end;
                           inc(y,h);
                     end;
                    end;
     wpmTransparent:
                    begin
                    bitblt(buffer.canvas.handle,0,0,width,height,canvas.handle,0,0,srccopy);
                    with finternalimage do
                         begin w:=width; h:=height; end;
                         y:=0;
                     while y<height do
                           begin
                           x:=0;
                           while x<width do
                                  begin
                                  DrawBitmapTransparent(buffer.Canvas,
                                  x, y, finternalimage, ftranscolor );
                                  inc(x,w);
                                  end;
                           inc(y,h);
                     end;
                    end;
     wpmNotebook   :begin
                    y:=0;
                    x:=0;
                    with buffer.canvas, fnboptions do
                         begin
                         brush.color:=papercolor;
                         pen.color:=papercolor;
                         rectangle(0,0,width,height);
                         pen.color:=highlight;
                         brush.color:=highlight;
                         while y<height do
                          begin
                          if odd(x) then
                             rectangle(xoffset,y,width-(xoffset+3),y+yoffset);

                          draw(5,y,finternalimage);
                          draw(width-(xoffset+4),y,finternalimage);
                          inc(y,yoffset+2);
                          inc(x);
                          end;
                         moveto(xoffset+6,0); lineto(xoffset+6,height);
                         moveto(width-(xoffset+6),0); lineto(width-(xoffset+6),height);
                         end;
                    end;
     wpmsawtooth   :begin
                    y:=0;
                    with buffer.canvas,fnboptions do
                         begin
                         brush.color:=fnboptions.papercolor;
                         pen.color:=fnboptions.papercolor;
                         rectangle(0,0,width,height);
                         brush.color:=fnboptions.highlight;
                         pen.color:=fnboptions.highlight;
                         while y<height do
                          begin
                          draw(0,y,finternalimage);
                          inc(y,yoffset);
                          end;
                         end;
                    end;
     end;
canvas.draw(0,0,buffer);
buffer.free;
end;

procedure TnonBMPOptions.setpcolor(value:tcolor);
begin
fpcolor:=value;
if (fowner.mode=wpmnotebook) or (fowner.mode=wpmsawtooth) then
begin
fowner.renderinternalimage;
fowner.invalidate;
end;
end;

procedure TnonBMPOptions.setback(value:tcolor);
begin
fback:=value;
if (fowner.mode=wpmnotebook) or (fowner.mode=wpmsawtooth) then
begin
fowner.renderinternalimage;
fowner.invalidate;
end;
end;

procedure TnonBMPOptions.setlcolor(value:tcolor);
begin
flcolor:=value;
if (fowner.mode=wpmnotebook) or (fowner.mode=wpmsawtooth) then
begin
fowner.renderinternalimage;
fowner.invalidate;
end;
end;

procedure TnonBMPOptions.sethole(value:tcolor);
begin
fhole:=value;
if (fowner.mode=wpmnotebook) or (fowner.mode=wpmsawtooth) then
begin
fowner.renderinternalimage;
fowner.invalidate;
end;
end;

procedure TnonBMPOptions.setxoffset(value:integer);
begin
fxoffset:=value;
if (fowner.mode=wpmnotebook) or (fowner.mode=wpmsawtooth) then
begin
fowner.renderinternalimage;
fowner.invalidate;
end;
end;

procedure TnonBMPOptions.setyoffset(value:integer);
begin
fyoffset:=value;
if (fowner.mode=wpmnotebook) or (fowner.mode=wpmsawtooth) then
begin
fowner.renderinternalimage;
fowner.invalidate;
end;
end;

end.
