unit bi_edits;
{ Highly enhanced & optimized
  version of a bitmap filled memo found in a article 'Glasnost'
  by Arne Schpers in the magazine C't, Issue #9 1996.
  }

interface
uses Windows, StdCtrls, Controls, Messages, Classes,
     Graphics, Forms, ExtCtrls, SysUtils,BI_Timer,bi_procs;

type

 TFillMode   = (fmNormal,fmFaded);
 TBorderStyleExt = (bseNone,bseSingle,bseFlat);

 TPublicEdit = class(TCustomEdit);
 tbackbase = class;
 TMemoPane = class(TCustomControl)
 private
   FOwner: tbackbase;
   Memo: TCustomEdit;
 protected
   procedure WMEraseBkGnd(var Msg: TWMEraseBkGnd); message WM_ERASEBKGND;
   procedure WMLButtonDown(var Msg: TWMMouse); message WM_LBUTTONDOWN;
   procedure Paint; override;
 public
   constructor Create(AOwner: TComponent); override;
 end;

 tbackbase = class(TCustomControl)
 private
   FLastCaretPos: TPoint;
   FFillMode:tfillmode;
   FPane: TMemoPane;
   FBackBMP,finternalimage, FBackBuf: TBitmap;
   FMemoMouseDown, FMemoCaretVisible: Boolean;
   FSelectionBrush: TBrush;
   SelStart, SelEnd, CharCount: LongInt;
   FShowSel, insel: Boolean;
   FOnChange, FOnClick, FOnDblClick: TNotifyEvent;
   FOnMouseDown, FOnMouseUp: TMouseEvent;
   FOnMouseMove: TMouseMoveEvent;
   ffadecolor, fcolor:tcolor;
   FOnDragDrop: TDragDropEvent; FOnDragOver: TDragOverEvent;
   FOnEndDrag: TEndDragEvent; FOnKeyDown, FOnKeyUp: TKeyEvent;
   FOnKeyPress: TKeyPressEvent;
   xoffset:integer;
   fborder:TBorderStyleExt;

   bars:tscrollstyle;
   caretposback:tpoint;
   inside:boolean;
   Procedure SetFillMode(Value:tfillmode);
   procedure setborder(value:TBorderStyleExt);
   procedure setcolor(value:tcolor);
   procedure CMMouseEnter(var Message: TMessage); message CM_MOUSEENTER;
   procedure CMMouseLeave(var Message: TMessage); message CM_MOUSELEAVE;
 protected
   procedure SelectBrush;
   procedure getfadedbackground;
   procedure drawborder;
   procedure PaintSelLine(var XPos: Integer; const YPos: Integer;
     var Line: String; const StrLen: Integer);
   procedure WMCommand(var Msg: TWMCommand); message WM_COMMAND;
   procedure WMEraseBkGnd(var Msg: TWMEraseBkGnd);  message WM_ERASEBKGND;
   procedure CMFontChanged(var Msg: TMessage); message CM_FONTCHANGED;
   procedure SetBackBMP(BMP: tbitmap);
   procedure ShowMemoCaret(Show: Boolean);
   function GetEditCtrl: TPublicEdit; virtual; abstract;
   procedure SetName(const Value: TComponentName); override;
   procedure MemoMouseDown(Sender: TObject; Button: TMouseButton;Shift: TShiftState; X, Y: Integer);
   procedure MemoMouseMove(Sender: TObject; Shift: TShiftState;X,Y: Integer);
   procedure MemoMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
   procedure SetOnChange(Value: TNotifyEvent);
   procedure MemoClick(Sender: TObject);
   procedure MemoDblClick(Sender: TObject);
   procedure MemoDragDrop(Sender, Source: TObject; X, Y: Integer);
   procedure MemoDragOver(Sender, Source: TObject; X, Y: Integer;
      State: TDragState; var Accept: Boolean);
   procedure MemoEndDrag(Sender, Target: TObject; X, Y: Integer);
   procedure MemoKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
   procedure MemoKeyPress(Sender: TObject; var Key: Char);
   procedure MemoKeyUp(Sender: TObject; var Key: Word;
      Shift: TShiftState);
   procedure SetHideSel(Value: Boolean); function GetHideSel: Boolean;
   procedure SetText(const Value: String); function GetText: String;
   procedure SetBorderStyle(Value: TBorderStyle);
   function GetBorderStyle: TBorderStyle;
   procedure CustomBounds(ALeft,ATop,AWidth,AHeight,PaneW,PaneH: Integer);
   procedure Paint; override;
   procedure PostCreate;
   procedure OnCaretTimer(Sender: TObject);
   procedure setfadecolor(Value:tcolor);
   property BorderStyle: TBorderStyle read GetBorderStyle
            write SetBorderStyle default bsnone;
 public
   constructor Create(AOwner: TComponent); override;
   destructor Destroy; override;
 published
   property Brush;
   Property Border: TBorderStyleExt read fborder write setborder;
   Property FillStyle:tfillmode read ffillmode write setfillmode;
   Property FadeColor:tcolor read ffadecolor write setfadecolor;
   Property Color:tcolor read fcolor write setcolor;
   property SelectionBrush: TBrush read FSelectionBrush  write FSelectionBrush;
   property Bitmap: tbitmap read FBackBMP write SetBackBMP;
   property OnChange: TNotifyEvent read FOnChange write SetOnChange;
   property OnMouseDown: TMouseEvent read FOnMouseDown write FOnMouseDown;
   property OnMouseMove: TMouseMoveEvent read FOnMouseMove write FOnMouseMove;
   property OnMouseUp: TMouseEvent read FOnMouseUp write FOnMouseUp;
   property OnClick: TNotifyEvent read FOnClick write FOnClick;
   property OnDblClick: TNotifyEvent read FOnDblClick write FOnDblClick;
   property OnDragDrop: TDragDropEvent read FOnDragDrop write FOnDragDrop;
   property OnDragOver: TDragOverEvent read FOnDragOver write FOnDragOver;
   property OnEndDrag: TEndDragEvent read FOnEndDrag write FOnEndDrag;
   property OnKeyDown: TKeyEvent read FOnKeyDown write FOnKeyDown;
   property OnKeyPress: TKeyPressEvent read FOnKeyPress write FOnKeyPress;
   property OnKeyUp: TKeyEvent read FOnKeyUp write FOnKeyUp;
   property HideSelection: Boolean read GetHideSel write SetHideSel default true;
   property Font;
   property DragCursor;
   property DragMode;
   property OnEnter;
   property OnExit;
   property Enabled;
   property TabOrder;
   property TabStop;
   property PopupMenu;
 end;

type
  TBI_Edit = class(tbackbase)
  private
    FEdit: TEdit;
    FOldSel: LongInt; FCaretInFront: Boolean;
  protected
   procedure SetBounds(ALeft, ATop, AWidth, AHeight: Integer); override;
   procedure Paint; override;
   function GetEditCtrl: TPublicEdit; override;
  public
   constructor Create(AOwner: TComponent); override;
   procedure Clear;
   procedure ClearSelection;
   procedure CopyToClipboard;
   procedure CutToClipboard;
   procedure PasteFromClipboard;
   procedure SelectAll;
  published
   property Text: String read GetText write SetText;
  end;

  TBI_Memo = class(tbackbase)
  private
    FMemo: TMemo;
  protected
   procedure SetScrollBars(const Value: TScrollStyle);
   function GetScrollBars: TScrollStyle;
   procedure setlmargin(value: integer);
   procedure SetLines(Value: TStrings); function GetLines: TStrings;
   procedure SetBounds(ALeft, ATop, AWidth, AHeight: Integer); override;
   procedure setww(value:boolean);
   function getww:boolean;
   function getmaxl:integer;
   procedure setmaxl(value:integer);
   procedure Paint; override;
   function GetEditCtrl: TPublicEdit; override;
  public
   constructor Create(AOwner: TComponent); override;
   procedure Clear;
   procedure ClearSelection;
   procedure CopyToClipboard;
   procedure CutToClipboard;
   procedure PasteFromClipboard;
   procedure SelectAll;
   procedure Undo;
  published
   property ScrollBars: TScrollStyle read GetScrollBars write SetScrollBars;
   property Lines: TStrings read GetLines write SetLines;
   property WordWrap: boolean read getww write setww default True;
  // property LeftMargin: integer read xoffset write setlmargin;
   property Align;
   property MaxLength:integer read getmaxl write setmaxl default 0;
  end;


implementation

const
  MEMO_CARETOFF = $1000; MEMO_CARETON = $1001;
  MEMO_SELCHANGED = $1002;

var CaretTimer: tbi_timer;

type
  TModifiedMemo = class(TMemo)
  protected
    procedure WndProc(var Msg: TMessage); override;
  end;

  TModifiedEdit = class(TEdit)
  protected
    procedure WndProc(var Msg: TMessage); override;
  end;

procedure ParentNotify(Ctrl: TWinControl; NotifyVal: Word);
begin
  Ctrl.Parent.Perform(WM_COMMAND,NotifyVal shl 16, Ctrl.Handle);
end;

procedure TModifiedMemo.WndProc(var Msg: TMessage);
var OldSelLen: Integer;
begin
oldsellen:=0; {stop compiler warnings}
  with Msg do
    if (Msg = WM_VSCROLL) and (wParam = SB_THUMBTRACK)
      then ParentNotify(Self,EN_VSCROLL)
    else if (Msg = WM_HSCROLL) and (wParam = SB_THUMBTRACK)
      then ParentNotify(Self,EN_HSCROLL)
    else if (Msg = WM_KEYDOWN) then
    begin
      OldSelLen := SelLength;
      ParentNotify(Self,MEMO_CARETOFF);
    end;
  inherited WndProc(Msg);
  if Msg.Msg = WM_KEYDOWN then
    if OldSelLen <> SelLength then ParentNotify(Self,MEMO_SELCHANGED)
      else ParentNotify(Self,MEMO_CARETON);
end;

procedure TModifiedEdit.WndProc(var Msg: TMessage);
begin
  if Msg.Msg = WM_KEYDOWN then ParentNotify(Self,MEMO_CARETOFF);
  inherited WndProc(Msg);
  if Msg.Msg = WM_KEYDOWN then ParentNotify(Self,MEMO_SELCHANGED);
end;

function GetEditHeight(const afont:tfont):integer;
var
  DC: HDC;
  SaveFont: HFONT;
  Metrics: TTextMetric;
begin
  DC := GetDC(0);
  SaveFont := SelectObject(DC, aFont.Handle);
  GetTextMetrics (DC, Metrics);
  SelectObject (DC, SaveFont);
  ReleaseDC (0, DC);
  Result := Metrics.tmHeight + 6;
end;

constructor TMemoPane.Create(AOwner: TComponent);
begin
  if AOwner is tbackbase then
  begin
    inherited Create(AOwner);
    FOwner := AOwner as tbackbase;
    Cursor := crIBeam;
  end
   else raise Exception.Create('TMemoPane: Owner <> tbi_memo/Edit');
end;

procedure TMemoPane.WMEraseBkGnd(var Msg: TWMEraseBkGnd);
begin
  Msg.Result := 1;
end;

procedure TMemoPane.WMLButtonDown(var Msg: TWMMouse);
begin
  if Memo <> nil then
  begin
    with Msg do Result := Memo.Perform(Msg,Keys,LongInt(Pos));
    if not Memo.Focused then Memo.SetFocus;
    Invalidate;
  end;
end;

procedure TMemoPane.Paint;
begin
  FOwner.Paint;
end;

procedure tbackbase.SetOnChange(Value: TNotifyEvent);
begin
FOnChange := Value;
end;

procedure tbackbase.CMFontChanged(var Msg: TMessage);
begin
  if FBackBuf <> nil then FBackBuf.Canvas.Font := Font;
  if GetEditCtrl <> nil then GetEditCtrl.Font := Font;
  Left := Left;
end;

procedure tbackbase.SetName(const Value: TComponentName);
var OldName: TComponentName;
begin
  OldName := Name; inherited SetName(Value);
  with GetEditCtrl do if Text = OldName then Text := Value;
end;

constructor tbackbase.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Ctl3D := false; ParentCtl3D := False;
  FBackBMP := tbitmap.Create;
  FBackBuf := TBitmap.Create;
  fcolor:=clwindow;
  fborder:=bseFlat;
  inside:=false;
  finternalimage := tbitmap.create;
  FSelectionBrush := TBrush.Create;
  controlstyle:=controlstyle+[csopaque];
  ffadecolor:=clbtnhighlight;
  with FSelectionBrush do
  begin
    Style := bsSolid; Color := clBlue;
  end;
  FPane := TMemoPane.Create(Self);
  with FPane do
  begin
    Parent := Self;
    OnMouseMove := MemoMouseMove;
    OnClick := MemoClick; OnDblClick := MemoDblClick;
    OnDragDrop := MemoDragDrop; OnDragOver := MemoDragOver;
    OnEndDrag := MemoEndDrag;
  end;
end;

procedure tbackbase.PostCreate;
begin
  with GetEditCtrl do
  begin
    Parent := Self;
    OnMouseDown := MemoMouseDown; OnMouseMove := MemoMouseMove;
    OnMouseUp := MemoMouseUp;
    OnClick := MemoClick; OnDblClick := MemoDblClick;
    OnDragOver := MemoDragOver; OnDragDrop := MemoDragDrop;
    OnEndDrag := MemoEndDrag;
    OnKeyPress := MemoKeyPress; OnKeyDown := MemoKeyDown;
    OnKeyUp := MemoKeyUp;
  end;
  FPane.Memo := GetEditCtrl; FPane.BringToFront;
  SetBounds(Left,Top,GetEditCtrl.Width,GetEditCtrl.Height);
end;

destructor tbackbase.Destroy;
begin
  FBackBMP.Free; FBackBuf.Free; FSelectionBrush.Free; finternalimage.free;
  inherited Destroy;
end;

constructor tbi_memo.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FMemo := TModifiedMemo.Create(Self);
  xoffset:=1;
  PostCreate;
  bars:=ssnone;
end;

function tbi_memo.GetEditCtrl: TPublicEdit;
begin
Result := TPublicEdit(FMemo);
end;

constructor TBI_Edit.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FEdit := TModifiedEdit.Create(Self);
  fedit.width:=121;
  width:=121;
  xoffset:=1;
  borderstyle:=bsnone;
  PostCreate;
  bars:=ssnone;
end;

function TBI_Edit.GetEditCtrl: TPublicEdit;
begin
Result := TPublicEdit(FEdit);
end;

procedure tbi_Edit.Clear;
begin
  fedit.clear;
  invalidate;
end;

procedure tbi_Edit.ClearSelection;
begin
  fedit.clearselection;
end;

procedure tbi_Edit.CopyToClipboard;
begin
  fedit.copytoclipboard;
end;

procedure tbi_Edit.CutToClipboard;
begin
  fedit.cuttoclipboard;
end;

procedure tbi_Edit.PasteFromClipboard;
begin
  fedit.pastefromclipboard;
end;

procedure tbi_Edit.SelectAll;
begin
  fedit.selectall;
end;

procedure tbi_memo.setlmargin(value: integer);
begin
if value<>xoffset then
   begin
   xoffset:=value;
   invalidate;
   end;
end;

procedure tbackbase.CustomBounds(ALeft,ATop,AWidth,AHeight,
  PaneW,PaneH: Integer);
var XYOff: Integer;
begin
  inherited SetBounds(ALeft,ATop,AWidth,AHeight);
  GetEditCtrl.SetBounds(0,0,AWidth,AHeight);
  //if GetEditCtrl.BorderStyle = bsNone then XYOff := 0 else XYOff := 1;
  XYOff:=0;
  FPane.SetBounds(XYOff,XYOff,PaneW-XYOff*2,PaneH-XYOff*2);
  FBackBuf.Canvas.Brush.Assign(Brush);
  if FPane.Width >= 0 then FBackBuf.Width := FPane.Width;
  if FPane.Height >= 0 then FBackBuf.Height := FPane.Height;
  Invalidate;
end;

Procedure tbackbase.SetFillMode(Value:tfillmode);
begin
ffillmode:=value;
if ffillmode=fmFaded then
   begin
   fbackbuf.canvas.brush.assign((parent as twincontrol).brush);
   fbackbuf.canvas.pen.style:=psclear;
   fbackbuf.canvas.rectangle(0,0,fbackbuf.width,fbackbuf.height);
   getfadedbackground;
   end;
invalidate;
end;

Procedure tbackbase.setfadecolor(Value:tcolor);
begin
if value<>ffadecolor then
begin
   ffadecolor:=value;
   getfadedbackground;
   invalidate;
end;
end;

procedure TBI_Edit.SetBounds(ALeft, ATop, AWidth, AHeight: Integer);
begin
  aheight:=geteditheight(font);
  CustomBounds(ALeft,ATop,AWidth,AHeight,AWidth,AHeight);
end;

procedure tbi_memo.SetBounds(ALeft, ATop, AWidth, AHeight: Integer);
var PaneW, PaneH, VScrollW, HScrollH: Integer;
begin
  HScrollH := GetSystemMetrics(SM_CYHSCROLL);
  VScrollW := GetSystemMetrics(SM_CXVSCROLL);
  PaneW := AWidth; PaneH := AHeight;
  case FMemo.ScrollBars of
    ssVertical: Dec(PaneW,VScrollW);
    ssHorizontal: Dec(PaneH,HScrollH);
    ssBoth:
           begin
              Dec(PaneW,VScrollW); Dec(PaneH,HScrollH);
           end;
  end;
  CustomBounds(ALeft,ATop,AWidth,AHeight,PaneW,PaneH);
end;

procedure tbackbase.WMCommand(var Msg: TWMCommand);
begin
  with Msg do
    if Ctl = GetEditCtrl.Handle then
    case NotifyCode of
      EN_UPDATE: ;
      EN_CHANGE:
        begin
          FPane.Invalidate;
          if Assigned(FOnChange) then
          begin
            Update; FOnChange(Self);
          end;
        end;
      EN_HSCROLL,
      EN_VSCROLL: Invalidate;
      EN_SETFOCUS:
        begin
         Invalidate;
         with CaretTimer do
         begin
          OnTimer       := OnCaretTimer;
          Enabled       := True;
          Interval      := GetCaretBlinkTime;
          threadpriority:= tpidle;
         end;
       end;
      EN_KILLFOCUS:
        begin
          ShowMemoCaret(False);
          if GetEditCtrl.HideSelection  then FPane.Invalidate;
        end;
      MEMO_CARETOFF: ShowMemoCaret(False);
      MEMO_CARETON: ShowMemoCaret(True);
      MEMO_SELCHANGED:  FPane.Invalidate;
      else
        raise Exception.Create(Name+': NotifyCode = '+
                               IntToStr(NotifyCode)+'?');
      end;
  inherited;
end;

procedure tbackbase.SetBackBMP(BMP: tbitmap);
begin
FBackBMP.Assign(BMP);
finternalimage.width:=0;
FPane.Invalidate;
end;

procedure tbackbase.SetBorder(value: TBorderStyleExt);
begin
if value <> fborder then
   begin
   fborder:=value;
   invalidate;
   end;
end;

procedure tbackbase.SetColor(value: Tcolor);
begin
if value <> fcolor then
   begin
   fcolor:=value;
   invalidate;
   end;
end;

procedure tbackbase.SetText(const Value: String);
begin
ShowMemoCaret(False);
GetEditCtrl.Text := Value;
end;

procedure tbackbase.CMMouseEnter(var Message: TMessage);
begin
inside:=true;
if fborder=bseflat then repaint;
end;

procedure tbackbase.CMMouseLeave(var Message: TMessage);
begin
inside:=false;
if fborder=bseflat then repaint;
end;

function tbackbase.GetText: String;
begin
Result := GetEditCtrl.Text;
end;

function tbackbase.GetBorderStyle: TBorderStyle;
begin
Result := GetEditCtrl.BorderStyle;
end;

procedure tbackbase.SetBorderStyle(Value: TBorderStyle);
begin
  GetEditCtrl.BorderStyle := Value; Left := Left;
end;

procedure tbackbase.SetHideSel(Value: Boolean);
begin
  GetEditCtrl.HideSelection := Value; FPane.Invalidate;
end;

function tbackbase.GetHideSel: Boolean;
begin
Result := GetEditCtrl.HideSelection;
end;

procedure tbi_memo.SetScrollBars(const Value: TScrollStyle);
begin
  FMemo.ScrollBars := Value;
  bars:=value;
  Left := Left;
end;

function tbi_memo.GetScrollBars: TScrollStyle;
begin
Result := FMemo.ScrollBars;
end;

procedure tbi_memo.Clear;
begin
  fmemo.clear;
  invalidate;
end;

procedure tbi_memo.ClearSelection;
begin
  fmemo.clearselection;
end;

procedure tbi_memo.CopyToClipboard;
begin
  fmemo.copytoclipboard;
end;

procedure tbi_memo.CutToClipboard;
begin
  fmemo.cuttoclipboard;
end;

procedure tbi_memo.PasteFromClipboard;
begin
  fmemo.pastefromclipboard;
end;

procedure tbi_memo.Undo;
begin
  sendmessage(fmemo.handle,EM_Undo,0,0)
end;

procedure tbi_memo.SelectAll;
begin
  fmemo.selectall;
end;

function tbi_memo.GetLines: TStrings;
begin
Result := FMemo.Lines;
end;

procedure tbi_memo.SetLines(Value: TStrings);
begin
FMemo.Lines := Value;
end;

procedure tbi_memo.setww(value:boolean);
begin
FMemo.wordwrap := Value;
end;

function tbi_memo.getww:boolean;
begin
result:=FMemo.Wordwrap;
end;

function tbi_memo.getmaxl:integer;
begin
result:=FMemo.maxlength;
end;

procedure tbi_memo.setmaxl(value:integer);
begin
FMemo.maxlength := Value;
end;

procedure tbackbase.MemoMouseDown(Sender: TObject; Button:
          TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  if Button = mbLeft then FMemoMouseDown := True;
  if Assigned(FOnMouseDown)
    then FOnMouseDown(Sender,Button,Shift,X,Y);
end;

procedure tbackbase.MemoMouseMove(Sender: TObject;
          Shift: TShiftState; X,Y: Integer);
begin
  if FMemoMouseDown then
  FPane.Invalidate;
  if Assigned(FOnMouseMove) then FOnMouseMove(Sender,Shift,X,Y);
end;

procedure tbackbase.MemoMouseUp(Sender: TObject;
          Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  if Button = mbLeft then FMemoMouseDown := False;
  if Assigned(FOnMouseUp) then FOnMouseUp(Sender,Button,Shift,X,Y);
end;

procedure tbackbase.MemoClick(Sender: TObject);
begin if Assigned(FOnClick) then FOnClick(Self); end;

procedure tbackbase.MemoDblClick(Sender: TObject);
begin if Assigned(FOnDblClick) then FOnDblClick(Self); end;

procedure tbackbase.MemoDragDrop(Sender, Source: TObject; X,
  Y: Integer);
begin if Assigned(FOnDragDrop) then FOnDragDrop(Sender,Source,X,Y);
end;

procedure tbackbase.MemoDragOver(Sender, Source: TObject;
          X, Y: Integer; State: TDragState; var Accept: Boolean);
begin
  if Assigned(FOnDragOver)
    then FOnDragOver(Sender,Source,X,Y,State,Accept);
end;

procedure tbackbase.MemoEndDrag(Sender, Target: TObject;
                                 X, Y: Integer);
begin
if Assigned(FOnEndDrag) then FOnEndDrag(Sender,Target,X,Y);
end;

procedure tbackbase.MemoKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
begin
if Assigned(FOnKeyDown) then FOnKeyDOwn(Sender,Key,Shift);
end;

procedure tbackbase.MemoKeyPress(Sender: TObject; var Key: Char);
begin
if Assigned(FOnKeyPress) then FOnKeyPress(Sender,Key);
end;

procedure tbackbase.MemoKeyUp(Sender: TObject; var Key: Word;
      Shift: TShiftState);
begin
if Assigned(FOnKeyUp) then FOnKeyUp(Sender,Key,Shift);
if key in [vk_left,vk_right,vk_home,vk_end] then repaint;
{don't want to have 2 carets in a line,
 but i confess this is a really sick workaround}
end;


procedure tbackbase.WMEraseBkGnd(var Msg: TWMEraseBkGnd);
begin
  Msg.Result := 1;
end;

procedure tbackbase.getfadedbackground;
var tmp:tbitmap;
begin
tmp:=tbitmap.create; tmp.width:=fbackbmp.width; tmp.height:=fbackbmp.height;
tmp.monochrome:=true;  tmp.assign(fbackbmp);
with finternalimage do
begin
assign(tmp);
Canvas.Brush.Color := color;
canvas.fillrect(rect(0,0,tmp.width,tmp.height));
DrawBitmapTransparent(finternalimage.Canvas,
           0, 0, tmp, clblack );
Canvas.Brush.Color := ffadecolor;{ffadecolor;  }
Canvas.BrushCopy
(Rect (0, 0, tmp.Width, tmp.Height), FinternalImage,
Rect (0, 0, tmp.Width, tmp.Height), clwhite);
end;
tmp.free;
end;

procedure tbackbase.drawborder;
var hscrollh,vscrollw:integer;
begin
   HScrollH := height-GetSystemMetrics(SM_CYHSCROLL)*booltoint((bars=ssboth) or (bars=sshorizontal));
   VScrollW := width-GetSystemMetrics(SM_CXVSCROLL)*booltoint((bars=ssboth) or (bars=ssvertical));;


   if (fborder=bseflat) then
   begin if (GetEditCtrl.Focused or (csdesigning in componentstate) or inside) then
       with fbackbuf.canvas do
       begin
       pen.style:=pssolid;
       pen.color:=clbtnshadow;
       moveto(0,0);
       lineto(VScrollW-1,0);
       pen.color:=clbtnhighlight;
       lineto(VScrollW-1,HScrollH-1);
       lineto(0,HScrollH-1);
       pen.color:=clbtnshadow;
       lineto(0,0);
       brush.color:=clbtnface;
       framerect(rect(1,1,VScrollW-1,HScrollH-1))
       end
   else
       with fbackbuf.canvas do
       begin
       brush.style:=bsclear;
       pen.color:=clbtnface;
       rectangle(0,0,VScrollW,HScrollH);
       rectangle(1,1,VScrollW-1,HScrollH-1)
       end
   end
   else if fborder=bseSingle then
   begin
   mydrawbuttonface(fbackbuf.canvas,rect(0,0,vscrollw,hscrollh),2,true,clbtnface,clbtnshadow);
   end;
end;

procedure tbackbase.Paint;
var x,y,h,w:integer;
begin
if (fbackbmp.width>0) and (fbackbmp.height>0) then
begin
case ffillmode of
fmNormal:
         with FBackBuf do
                         begin
                         w:=fbackbmp.width; h:=fbackbmp.height;
                                                y:=0;
                                            while y<height do
                                                  begin
                                                  x:=0;
                                                  while x<width do
                                                         begin
                                                         canvas.draw(x,y,fbackbmp);
                                                         inc(x,w);
                                                         end;
                                                  inc(y,h);
                                            end;
         end;
fmfaded:
        with FBackBuf do
                         begin
                         if finternalimage.width=0 then
                                                   getfadedbackground;
                         w:=finternalimage.width; h:=finternalimage.height;
                                                y:=0;
                                            while y<height do
                                                  begin
                                                  x:=0;
                                                  while x<width do
                                                         begin
                                                         canvas.draw(x,y,finternalimage);
                                                         inc(x,w);
                                                         end;
                                                  inc(y,h);
                                            end;
         end;

end;
end
else with fbackbuf.canvas do begin
pen.color:=fcolor;
brush.color:=fcolor;
rectangle(0,0,width,height);
end;

drawborder;


  SelStart := GetEditCtrl.SelStart;
  SelEnd := SelStart+GetEditCtrl.SelLength;
  FShowSel := (SelStart <> SelEnd) and
              (GetEditCtrl.Focused or not HideSelection);
end;

procedure tbackbase.PaintSelLine(var XPos: Integer; const YPos: Integer;
    var Line: String; const StrLen: Integer);
var PartStr: String;
begin
  PartStr := Copy(Line,1,StrLen); System.Delete(Line,1,StrLen);
  with FBackBuf.Canvas do
  begin
    TextOut(XPos,YPos,PartStr);
    Inc(XPos,TextWidth(PartStr)); Inc(CharCount,Length(PartStr));
  end;
end;

procedure tbackbase.SelectBrush;
begin
  insel := (SelStart <= CharCount) and (SelEnd > CharCount) and
                 FShowSel;
  with FBackBuf.Canvas.Brush do
    if insel then Assign(FSelectionBrush) else Style := bsClear;
end;

procedure TBI_Edit.Paint;
var XPos, CaretIndex: Integer; CaretPos: TPoint; Line: String;
begin
  inherited Paint;
  Line := FEdit.Text;
  if SelStart <> LoWord(FOldSel) then FCaretInFront := True;
  if SelEnd <> HiWord(FOldSel) then FCaretInFront := False;
  FOldSel := MakeLong(SelStart,SelEnd);
  if FCaretInFront then CaretIndex := SelStart
                   else CaretIndex := SelEnd;

  if FEdit.Focused then
  begin
    GetCaretPos(CaretPos);
    FLastCaretPos := CaretPos;
  end
  else CaretPos := FLastCaretPos;
  CharCount := 0;
  while (FBackBuf.Canvas.TextWidth(Copy(Line,CharCount+1,
                                   CaretIndex-CharCount))
    > CaretPos.X) do Inc(CharCount);
  Delete(Line,1,CharCount);
  SelectBrush; XPos := 4;
  if fbackbmp.width<1 then xpos:=3;
  if FShowSel then
  begin
    if not insel then
    begin
      PaintSelLine(XPos,2,Line,SelStart-CharCount); SelectBrush;
    end;
    PaintSelLine(XPos,2,Line,SelEnd-CharCount); SelectBrush;
  end;
  PaintSelLine(XPos,2,Line,255);
  Canvas.Draw(FPane.Left,FPane.Top,FBackBuf);
  ShowMemoCaret(True);
end;

procedure Tbi_memo.Paint;
var XPos, YPos, StartLine, LineHeight, LineLen: Integer;
    Line: String;
begin
  inherited Paint;
  LineHeight := FBackBuf.Canvas.TextHeight('W');
  StartLine := SendMessage(FMemo.Handle,EM_GETFIRSTVISIBLELINE,0,0);
  CharCount := FMemo.Perform(EM_LINEINDEX,StartLine,0);
  SelectBrush;
  YPos := 0;
  while (StartLine < FMemo.Lines.Count) and
        (YPos+LineHeight < FBackBuf.Height) do
  begin
    XPos := -GetScrollPos(FMemo.Handle,SB_HORZ)+xoffset+2;
    Line := FMemo.Lines[StartLine];
    LineLen := Length(Line)+2;
    if (insel and (SelEnd < CharCount+LineLen)) or
       (not insel and (SelStart >= CharCount) and
        (SelStart < CharCount+LineLen)) then
    begin
      if not insel then
        PaintSelLine(XPos,YPos+1,Line,SelStart-CharCount); SelectBrush;
      PaintSelLine(XPos,YPos+1,Line,SelEnd-CharCount); SelectBrush;
    end;
    PaintSelLine(XPos,YPos+1,Line,255); Inc(CharCount,2);
    Inc(YPos,LineHeight); Inc(StartLine);
  end;
  Canvas.Draw(0,0,FBackBuf);
  ShowMemoCaret(True);
end;

procedure tbackbase.ShowMemoCaret(Show: Boolean);
begin
  if (FMemoCaretVisible = Show) or
     (Show and not GetEditCtrl.Focused) then Exit;
  FMemoCaretVisible := Show;
  with canvas do
  begin
  Pen.Mode := pmXor; Pen.Color := clWhite; Pen.Width := 2;
  caretposback:=flastcaretpos;
  if Show then
     GetCaretPos(FLastCaretPos);
  if caretposback.y<>FLastCaretPos.y then {no two carets in dif lines}
     begin
     MoveTo(caretposback.X+xoffset+2,caretposback.Y+1);
     LineTo(caretposback.X+xoffset+2,caretposback.Y +
                FBackBuf.Canvas.TextHeight('X'));
     end;
  MoveTo(FLastCaretPos.X+xoffset+2,FLastCaretPos.Y+1);
  LineTo(FLastCaretPos.X+xoffset+2,FLastCaretPos.Y +
                FBackBuf.Canvas.TextHeight('X'));
  end;
end;

procedure tbackbase.OnCaretTimer(Sender: TObject);
begin
ShowMemoCaret(not FMemoCaretVisible);

end;

begin
  CaretTimer := TBI_Timer.Create(Application);
end.
