

unit cptbtn;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Forms,
  bi_timer,bi_tools,bi_procs;

type
  EOwnerError = class(Exception);
  EtoomanyError = class(Exception);
  TCPTbtnNotifyEvent = procedure ( Sender: TObject ) of object;

  TCaptionButton = class(TComponent)
  {Credits:
  the basic code was ripped from Sean Hsieh's winroll.pas
  and modified to draw anything...in a rather simple way, as
  I confess, so any suggestions on improvement would be great!}
  private
    DefWinProc: TFarProc;
    DefWinProcInstance: Pointer;
    FEnabled: Boolean;
    fonclick: TCPTbtnNotifyEvent;
    FOnFormDestroy: TNotifyEvent;
    FVisible: Boolean;
    FGlyph:tbitmap;
    ButtonArea: TRect;
    DrawPushed: Boolean;
    Pushed: Boolean;
    RestoreWndProc: Boolean;
    rgn: HRGN;
    Stop: Boolean;
    fstretch:boolean;
    myID:integer;
    hinttimer:tbi_timer;
    lastpos:tpoint;
    fhint:string;
    fshowhint:boolean;
    thehint:tinstanthint;
    fdown:boolean;

    fnumglyphs,gwidth,transclr:integer;
    procedure CalcArea;
    procedure OnButDestroy( Sender: TObject );
    procedure PaintButton;
    procedure SetEnabled( Value: Boolean );
    procedure SetVisible( Value: Boolean );
    procedure SetStretch( Value: Boolean );
    procedure SetDown( Value: Boolean );
    procedure SetGlyph( Value: tbitmap );
    procedure setnumglyphs(value:integer);
    procedure WinProc(var Message: TMessage);
    procedure CheckHint(sender:tobject);
    procedure loaded; override;
    procedure Getglyph(const no:byte;var c_glyph:tbitmap);
    procedure GetglyphStretched(const no:byte;var c_glyph:tbitmap;w,h:integer);
  public
    { Public declarations }
    constructor Create(Owner: TComponent); override;
    destructor Destroy; override;
  published
    { Published declarations }
    property Enabled: Boolean read FEnabled write SetEnabled;
    property Visible: Boolean read FVisible write SetVisible default True;
    property OnClick: TCPTbtnNotifyEvent read fonclick write fonclick;
    property NumGlyphs: integer read fnumglyphs write setnumglyphs;
    property Glyph: TBitmap read fglyph write setglyph;
    property Stretch:boolean read fstretch write setstretch;
    property Hint:string read fhint write fhint;
    property Down:boolean read fdown write setdown;
    property ShowHint:boolean read fshowhint write fshowhint;
  end;

procedure Register;

var
  si: UINT;

implementation

procedure Register;
begin
  RegisterComponents('Samples', [TCaptionButton]);
end;

var numbuttons:integer=0;
    ctrlarray:array[0..10] of tcaptionbutton;

constructor TCaptionButton.Create(Owner: TComponent);
begin
  inherited;
  fglyph:=tbitmap.create;
  fnumglyphs:=1;
  fdown:=false;
  gwidth:=0;
  fstretch:=false;
  FVisible := True;
  DrawPushed := False;
  Pushed := False;
  rgn := 0;
  fenabled:=true;
  if not (Owner is TForm) then
    raise EOwnerError.Create('Class of the owner is not a TForm!');

  inc(numbuttons);
  if numbuttons>10 then
    raise EtoomanyError.Create('Only 10 CaptionButtons supported!');
  myID:=numbuttons;
  ctrlarray[myID]:=self;
  with TForm(Owner) do
  if not (csDesigning in ComponentState) then
  begin
    DefWinProcInstance := MakeObjectInstance(WinProc);
    DefWinProc := Pointer(SetWindowLong(Handle, GWL_WNDPROC, Longint(DefWinProcInstance)));
    CalcArea;
    FOnFormDestroy := OnDestroy;
    OnDestroy := OnButDestroy;
  end;
  fhint:='';
  fshowhint:=false;
  hinttimer:=tbi_timer.create(self);
  hinttimer.enabled:=false;
  hinttimer.interval:=application.hintpause;
  hinttimer.threadpriority:=tplowest;
  hinttimer.ontimer:=checkhint;
  thehint:=tinstanthint.create(self);
end;

destructor TCaptionButton.Destroy;
begin
dec(numbuttons);
  if not (csDesigning in ComponentState) and not RestoreWndProc then
  begin
  //SetWindowLong(TForm(Owner).Handle, GWL_WNDPROC, Longint(DefWinProc));
    FreeObjectInstance(DefWinProcInstance);
  end;
  if rgn <> 0 then
    DeleteObject( rgn );
  fglyph.free;
  hinttimer.enabled:=false;
  hinttimer.free;
  thehint.free;
  inherited destroy;
end;

procedure TCaptionButton.OnButDestroy( Sender: TObject );
begin
 dec(numbuttons);
 if numbuttons=0 then
 begin
  if Assigned( FOnFormDestroy ) then
    FOnFormDestroy( Sender );
  SetWindowLong(TForm(Owner).Handle, GWL_WNDPROC, Longint(DefWinProc));
  FreeObjectInstance(DefWinProcInstance);
  RestoreWndProc := True;
 end;
end;

procedure TCaptionButton.checkhint(sender:tobject);
var _pos:tpoint;
begin
hinttimer.enabled:=false;
getcursorpos(_pos);
if (_pos.x=lastpos.x) AND (_pos.y=lastpos.y) then
   begin
   thehint.caption:=fhint;
   thehint.rectleft:=_pos.x+16;
   thehint.recttop:=_pos.y+16;
   thehint.execute;
   end;
end;

procedure TCaptionButton.CalcArea;
var
  x1, y1, x2, y2, x3: Integer;
  Icons: TBorderIcons;
  Style: TFormBorderStyle;
begin
x1:=-1; y1:=-1; x2:=-1; y2:=-1;
  with TForm(Owner) do
  begin
    if csDesigning in ComponentState then
    begin
      Icons := [biSystemMenu, biMinimize, biMaximize];
      Style := bsSizeable;
    end
    else
    begin
      Icons := BorderIcons;
      Style := BorderStyle;
    end;
  end;
  if Style in [bsSizeToolWin, bsToolWindow] then
  begin
    if Style = bsToolWindow then
      x2 := GetSystemMetrics(SM_CXFIXEDFRAME) + 2
    else
      x2 := GetSystemMetrics(SM_CXSIZEFRAME) + 2;
    if biSystemMenu in Icons then
      Inc(x2, GetSystemMetrics(SM_CXSMSIZE) + 2);
    if Style = bsToolWindow then
      y1 := GetSystemMetrics(SM_CYFIXEDFRAME) + 2
    else
      y1 := GetSystemMetrics(SM_CYSIZEFRAME) + 2;
    y2 := y1 + GetSystemMetrics(SM_CYSMSIZE) - 4;
    x2 := TForm(Owner).Width - x2;
    x1 := x2 - GetSystemMetrics(SM_CXSMSIZE) + 2;
  end
  else
  begin
    if Style in [bsSingle, bsSizeable, bsDialog] then
    begin
      if Style = bsSingle then
        x2 := GetSystemMetrics(SM_CYFIXEDFRAME) + 2
      else
        x2 := GetSystemMetrics(SM_CXSIZEFRAME) + 2;
      if biSystemMenu in Icons then
      begin
        Inc(x2, GetSystemMetrics(SM_CXSIZE) + 2);
        if (Style <> bsDialog) and (Icons * [biMinimize, biMaximize] <> []) then
          Inc(x2, GetSystemMetrics(SM_CXSIZE) * 2 - 6)
        else if biHelp in Icons then
          Inc(x2, GetSystemMetrics(SM_CXSIZE) - 4);
      end;
      if Style in [bsSingle, bsDialog] then
        y1 := GetSystemMetrics(SM_CYFIXEDFRAME) + 2
      else
        y1 := GetSystemMetrics(SM_CYSIZEFRAME) + 2;
      y2 := y1 + GetSystemMetrics(SM_CYSIZE) - 4;
      x2 := TForm(Owner).Width - x2;
      x1 := x2 - GetSystemMetrics(SM_CXSIZE) + 2;
    end;
  end;
  if (x1<>-1) and (x2<>-1) and (y1<>-1) and (y2<>-1) then
  begin
  x3:=(x2-x1)*(myID-1);
  SetRect(ButtonArea, x1-x3, y1, x2-x3, y2);
  end;
end;

procedure TCaptionButton.GetglyphStretched(const no:byte;var c_glyph:tbitmap;w,h:integer);
begin
c_glyph.width:=w;c_glyph.height:=h;
case no of
     1: stretchblt( c_glyph.canvas.handle,0,0,w,h,
              fglyph.canvas.handle,0,0,gwidth,fglyph.height,srccopy);
     2: if fnumglyphs>1 then
        stretchblt( c_glyph.canvas.handle,0,0,w,h,
              fglyph.canvas.handle,gwidth,0,gwidth*2,fglyph.height,srccopy)
      else
      stretchblt( c_glyph.canvas.handle,0,0,w,h,
              fglyph.canvas.handle,0,0,gwidth,fglyph.height,srccopy);
     3: if fnumglyphs>2 then
      stretchblt( c_glyph.canvas.handle,0,0,w,h,
              fglyph.canvas.handle,gwidth*2,0,gwidth*3,fglyph.height,srccopy)
      else
      stretchblt( c_glyph.canvas.handle,0,0,w,h,
              fglyph.canvas.handle,0,0,gwidth,fglyph.height,srccopy);
      end;
//if fgray then mygrayedbmp(c_glyph,clbtnhighlight,clbtnshadow);
end;

procedure TCaptionButton.Getglyph(const no:byte;var c_glyph:tbitmap);
begin
c_glyph.width:=gwidth;c_glyph.height:=fglyph.height;
case no of
     1: bitblt( c_glyph.canvas.handle,0,0,c_glyph.width,c_glyph.height,
              fglyph.canvas.handle,0,0,srccopy);
     2: if fnumglyphs>1 then
      bitblt( c_glyph.canvas.handle,0,0,c_glyph.width,c_glyph.height,
              fglyph.canvas.handle,gwidth,0,srccopy)
      else
      bitblt( c_glyph.canvas.handle,0,0,c_glyph.width,c_glyph.height,
              fglyph.canvas.handle,0,0,srccopy);
     3: if fnumglyphs>2 then
      bitblt( c_glyph.canvas.handle,0,0,c_glyph.width,c_glyph.height,
              fglyph.canvas.handle,gwidth*2,0,srccopy)
      else
      bitblt( c_glyph.canvas.handle,0,0,c_glyph.width,c_glyph.height,
              fglyph.canvas.handle,0,0,srccopy);
      end;
//if fgray then mygrayedbmp(c_glyph,clbtnhighlight,clbtnshadow);
end;

procedure TCaptionButton.PaintButton;
var
  _hdc: HDC;
  st: UINT;
  c: Integer;
  b: HBRUSH;
            procedure drawbutton(enabled:boolean);
            var offs_,x1, x2, y1, y2: Integer;
                tmp:tbitmap;
                canv:tcanvas;
            begin
            canv:=tcanvas.create;
            canv.handle:=_hdc;
            offs_:=0;
            if drawpushed or fdown then
               offs_:=1;
            x1:=buttonarea.left+1+offs_;
            x2:=buttonarea.right-buttonarea.left-4+offs_;
            y1:=buttonarea.top+offs_+1;
            y2:=buttonarea.bottom-buttonarea.top-4+offs_;

            if gwidth>0 then
            begin
            tmp:=tbitmap.create;
            if stretch then
            if not fdown then getglyphstretched(1,tmp,x2,y2) else getglyphstretched(3,tmp,
            x2,y2)
            else
            begin
            if not fdown then getglyph(1,tmp) else getglyph(3,tmp);
            end;
            DrawBitmapTransparent(canv,
            x1, y1, tmp, transclr );
            tmp.free;
            end;
            canv.free;
            end;
begin
  if not (csDesigning in ComponentState) then
    with TForm(Owner) do
      if not (BorderStyle = bsNone) then
      begin
        _hdc := GetWindowDC( Handle );
        if FVisible then
        begin
          if DrawPushed then
            st := DFCS_PUSHED
          else
            st := 0;
          DrawFrameControl( _hdc, ButtonArea, DFC_BUTTON, DFCS_BUTTONPUSH or st );
          if FEnabled then
          begin
            DrawButton( True );
          end
          else
          begin
            DrawButton( False );
          end;
        end
        else
        begin
          if Active then
            c := COLOR_ACTIVECAPTION
          else
            c := COLOR_INACTIVECAPTION;
          b := CreateSolidBrush( GetSysColor( c ) );
          FillRect( _hdc, ButtonArea, b );
          DeleteObject( b );
        end;
        ReleaseDC( Handle, _hdc );
      end;
end;

procedure TCaptionButton.SetEnabled( Value: Boolean );
var i:integer;
begin
  if Value <> FEnabled then
  begin
    FEnabled := Value;
    for i:=0 to numbuttons do if ctrlarray[i]<>nil then ctrlarray[i].paintbutton;
  end;
end;

procedure TCaptionButton.Setstretch( Value: Boolean );
var i:integer;
begin
  if Value <> fstretch then
  begin
    fstretch := Value;
    for i:=0 to numbuttons do if ctrlarray[i]<>nil then ctrlarray[i].paintbutton;
  end;
end;

procedure TCaptionButton.setdown( Value: Boolean );
var i:integer;
begin
  if Value <> fdown then
  begin
    fdown := Value;
    for i:=0 to numbuttons do if ctrlarray[i]<>nil then ctrlarray[i].paintbutton;
  end;
end;

procedure TCaptionButton.SetGlyph( Value: Tbitmap );
var i:integer;
begin
fglyph.assign(value);
if (value<>nil) and (value.height<>0) then
   begin
      fglyph.assign(value);
      transclr:=value.transparentcolor;
      if Value.Width mod Value.Height = 0 then
         fnumglyphs := Value.Width div Value.Height;
      if fnumglyphs>0 then
      gwidth:=fglyph.width div fnumglyphs
      else
      gwidth:=fglyph.width;
   end
else
    begin
    gwidth:=0;
    fnumglyphs:=0;
    end;

    for i:=0 to numbuttons do if ctrlarray[i]<>nil then ctrlarray[i].paintbutton;

end;

procedure TCaptionButton.SetVisible( Value: Boolean );
var i:integer;
begin
  if Value <> FVisible then
  begin
    FVisible := Value;
    CalcArea;
    for i:=0 to numbuttons do if ctrlarray[i]<>nil then ctrlarray[i].paintbutton;
  end;
end;

procedure TCaptionButton.Setnumglyphs( Value: Integer );
var i:integer;
begin
  if (Value <> fnumglyphs) AND (Value<>0) then
  begin
    fnumglyphs := Value;
    for i:=0 to numbuttons do if ctrlarray[i]<>nil then ctrlarray[i].paintbutton;
  end;
end;

procedure TCaptionButton.WinProc(var Message: TMessage);
var
  v: Boolean;
  i:integer;
  procedure Default;
  begin
    with Message do
      Result := CallWindowProc(DefWinProc, TForm(Owner).Handle, Msg, wParam, lParam);
  end;

  function InArea( InClient: Boolean ): Boolean;
  var
    p: TPoint;

  begin
    p.x := Message.lParamLo;
    p.y := Smallint(Message.lParamHi);
    if InClient then
      ClientToScreen( TForm(Owner).Handle, p );
    Dec( p.x, TForm(Owner).Left );
    Dec( p.y, TForm(Owner).Top );
    Result := PtInRect( ButtonArea, p );
  end;

begin
  if not FVisible then
    Default
  else
  with Message do
    case Msg of
      WM_MOVE:
      begin
        Default;
        Stop := True;
      end;
      WM_SIZE, WM_WINDOWPOSCHANGED:
      begin
        Default;
        v := FVisible;
        FVisible := False;
        for i:=0 to numbuttons do if ctrlarray[i]<>nil then ctrlarray[i].paintbutton;
        CalcArea;
        FVisible := v;
        for i:=0 to numbuttons do if ctrlarray[i]<>nil then ctrlarray[i].paintbutton;
        Stop := False;
      end;
      WM_MOUSEMOVE:
      begin
        if Pushed then
        begin
          if not InArea( True ) then
          begin
            if DrawPushed then
            begin
              DrawPushed := False;
              if FEnabled then
                for i:=0 to numbuttons do if ctrlarray[i]<>nil then ctrlarray[i].paintbutton;
            end;
          end
          else
          begin
            if not DrawPushed then
            begin
              DrawPushed := True;
              if FEnabled then
                for i:=0 to numbuttons do if ctrlarray[i]<>nil then ctrlarray[i].paintbutton;
            end;
          end;
          Result := 1;
        end
        else
          default;
        end;
      WM_LBUTTONUP, WM_LBUTTONDBLCLK:
      begin
        DrawPushed := False;
        if Pushed then
        begin
          if InArea( True ) then
          begin
            if (fenabled) AND Assigned( FOnClick ) then FOnClick( Self );
            for i:=0 to numbuttons do if ctrlarray[i]<>nil then ctrlarray[i].paintbutton;
          end
          else if FEnabled then
            for i:=0 to numbuttons do if ctrlarray[i]<>nil then ctrlarray[i].paintbutton;
          Result := 1;
        end
        else
          Default;
        Pushed := False;
        ReleaseCapture;
      end;
      WM_NCLBUTTONDOWN, WM_NCLBUTTONDBLCLK:
      begin
        if InArea( False ) then
        begin
          SetCapture( TForm(Owner).Handle );
          DrawPushed := True;
          Pushed := True;
          if FEnabled then
            for i:=0 to numbuttons do if ctrlarray[i]<>nil then ctrlarray[i].paintbutton;
          Result := 1;
        end;
        if Msg = WM_NCLBUTTONDOWN then
          Default;
      end;
      WM_SETTINGCHANGE:
      begin
        Default;
        setglyph(fglyph);
        CalcArea;
        for i:=0 to numbuttons do
        if ctrlarray[i]<>nil then ctrlarray[i].paintbutton;
      end;
      WM_NCACTIVATE, WM_NCPAINT:
      begin
        Default;
        for i:=0 to numbuttons do
        if ctrlarray[i]<>nil then ctrlarray[i].paintbutton;
      end;
      wm_ncmousemove:
      begin
      if inarea(false) then
         begin
         getcursorpos(lastpos);
         if (fhint<>'') and (fshowhint) then hinttimer.enabled:=true;
         end
         else
         default;
      end
      else
        Default;
    end;
end;

procedure TCaptionButton.loaded;
begin
inherited loaded;
setglyph(fglyph);
end;

end.
